package com.example.jsfUnitServerSessionNpeExample;

import java.io.File;
import java.io.IOException;

import javax.faces.component.UIComponent;

import org.jboss.arquillian.api.Deployment;
import org.jboss.arquillian.testng.Arquillian;
import org.jboss.jsfunit.jsfsession.JSFClientSession;
import org.jboss.jsfunit.jsfsession.JSFServerSession;
import org.jboss.jsfunit.jsfsession.JSFSession;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.testng.Assert;
import org.testng.annotations.Test;

public class TestControllerTest extends Arquillian {
	private static final String VIEW_URL = "/testController.jsf";
	private JSFSession session = null;
	private JSFServerSession server = null;
	private JSFClientSession client = null;

	@Deployment
	public static WebArchive createTestArchive() {
		return ShrinkWrap
				.create(WebArchive.class, "test.war")
				.addClasses(TestController.class)
				.setWebXML(new File("src/main/webapp/WEB-INF/web.xml"))
				.addResource(
						new File("src/main/webapp", "testController.xhtml"))
				.addWebResource(
						new File("src/main/webapp/WEB-INF/faces-config.xml"),
						"faces-config.xml");
	}

	// @BeforeMethod Doesn't work see https://jira.jboss.org/browse/ARQ-56
	@SuppressWarnings("unused")
	private void setUp() throws IOException {
	}

	@Test
	void testControllerRenderATest() throws IOException {
		UIComponent component = getServer().findComponent("test_controller_a");

		Assert.assertTrue(component.isRendered());
	}

	@Test
	void testControllerRenderBTest() throws IOException {
		UIComponent component = getServer().findComponent("test_controller_b");

		Assert.assertTrue(component.isRendered());
	}

	@Test
	void testControllerRenderCTest() throws IOException {
		UIComponent component = getServer().findComponent("test_controller_c");

		Assert.assertTrue(component.isRendered());
	}

	@Test
	void testControllerRenderDTest() throws IOException {
		UIComponent component = getServer().findComponent("test_controller_d");

		Assert.assertTrue(component.isRendered());
	}

	@Test
	void testControllerRenderETest() throws IOException {
		UIComponent component = getServer().findComponent("test_controller_e");

		Assert.assertTrue(component.isRendered());
	}

	@Test
	void testControllerValueATest() throws IOException {
		Assert.assertEquals(
				getServer().getManagedBeanValue("#{testController.a}"), "a");
	}

	@Test
	void testControllerValueBTest() throws IOException {
		Assert.assertEquals(
				getServer().getManagedBeanValue("#{testController.b}"), "b");
	}

	@Test
	void testControllerValueCTest() throws IOException {
		Assert.assertEquals(
				getServer().getManagedBeanValue("#{testController.c}"), "c");
	}

	@Test
	void testControllerValueDTest() throws IOException {
		Assert.assertEquals(
				getServer().getManagedBeanValue("#{testController.d}"), "d");
	}

	@Test
	void testControllerValueETest() throws IOException {
		Assert.assertEquals(
				getServer().getManagedBeanValue("#{testController.e}"), "e");
	}
	
	@Test
	void testFailTest() throws IOException {
		Assert.fail();
	}

	private JSFSession getSession() throws IOException {
		if (null == session) {
			session = new JSFSession(VIEW_URL);
		}
		return session;
	}

	private JSFServerSession getServer() throws IOException {
		if (null == server) {
			server = getSession().getJSFServerSession();
		}
		return server;
	}

	@SuppressWarnings("unused")
	private JSFClientSession getClient() throws IOException {
		if (null == client) {
			client = getSession().getJSFClientSession();
		}
		return client;
	}
}
